from requests.utils import quote
import http.client
import json

class EnamineSession:
    def __init__(self, base_url='www.enaminestore.com'):
        """
        Sets up a session to search Enamine REAL space with. There are two options available: substructure search, or
        similarity search. After setting up a session (e.g. session = EnamineSession()), run a similarity search with
        session.similarity_search(smiles, threshold) or a substructure search with session.substructure_search(smiles)

        :param base_url: str
            the base url for the online enamine store. Default = www.enaminestore.com
        """

        self.base_url = base_url
        self.conn = http.client.HTTPSConnection(base_url)
        self.headers = {
            'Connection': 'keep-alive',
            'sec-ch-ua': '"Chromium";v="88", "Google Chrome";v="88", ";Not A Brand";v="99"',
            'Accept': 'application/json',
            'X-Prototype-Version': '1.7',
            'X-Requested-With': 'XMLHttpRequest',
            'sec-ch-ua-mobile': '?0',
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.150 Safari/537.36',
            'Content-type': 'application/x-www-form-urlencoded; charset=UTF-8',
            'Origin': 'https://www.enaminestore.com',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Dest': 'empty',
            'Referer': 'https://www.enaminestore.com/search',
            'Accept-Language': 'en-GB,en-US;q=0.9,en;q=0.8',
            'Content-Type': 'text/plain'
        }

    def run_request(self, search_ext, full_str):
        """
        Runs the actual requests from similarity search or substructure search.

        :param search_ext: str
            search extension on top of base url in class __init__
        :param full_str: str
            the encoded string to send as raw-data to the url
        :return: result
        """
        self.conn.request("POST", search_ext, full_str, self.headers)
        res = self.conn.getresponse()
        data = res.read()
        result = data.decode("utf-8")
        try:
            r2 = json.loads(result)
            r3 = json.loads(r2['result'][0])
            list_of_mols = r3['data']
            return list_of_mols
        except:
            return result

    def similarity_search(self, smiles, threshold, search_ext="/search.realsearch:searchrealsendrequest"):
        """
        Performs a similarity search against Enamine Real based on a smiles string and similarity threshold
        :param smiles: str
            SMILES string to search for similar molecules of
        :param threshold: float
            value between 0 and 1, where 0 is no similarity, and 1 is an exact match
        :param search_ext:
        :return:
        """
        json_data = quote(f'{{"search":{{"query":"{smiles}","similarityThreshold":"{str(threshold)}"}}}}')
        full_str = f'controlPath=sim&jsonData={json_data}&PageSize=10000&reqMethod=POST'

        return self.run_request(search_ext=search_ext, full_str=full_str)

    def substructure_search(self, smiles, search_ext="/search.realsearch:searchrealsendrequest"):
        """
        Performs a substructure search based on an input molecule by smiles
        :param smiles: str
            SMILES string of substructure to search for
        :param search_ext:
        :return:
        """

        json_data = quote(f'{{"search":"{smiles}"}}')
        full_str = f'controlPath=sub&jsonData={json_data}&PageSize=10000&reqMethod=POST'

        return self.run_request(search_ext=search_ext, full_str=full_str)



# To use a consistent encoding
from codecs import open
from os import path

# Always prefer setuptools over distutils
from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))

long_description = "Wrapper around similarity and substructure searching functionality on enaminestore.com"

print(find_packages(exclude=["contrib", "docs", "tests"]))
setup(
    name="ers",
    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version="1.0.2",
    description="Wrapper around similarity and substructure searching functionality on enaminestore.com",
    long_description=long_description,
    # The project's main homepage.
    url="https://github.com/xchem/enamine-real-search.git",
    # Author details
    author="Rachael Skyner",
    author_email="rachael.skyner@diamond.ac.uk",
    # Choose your license
    license="Apache 2.0",
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 4 - Beta",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: Apache Software License",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3.6",
    ],
    # What does your project relate to?
    keywords="",
    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=["contrib", "docs", "tests"]),
    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
       py_modules=["search"],
    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['requests'],
    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    # extras_require={"dev": ["check-manifest"], "test": ["coverage"]},
    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={},
    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    #    data_files=[('my_data', ['data/data_file'])],
    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        "console_scripts": [
        ]
    },
)
